#include <ch.h>
#include <hal.h>
#include "buttons.h"

static btn_hndlr btnh[4];

/* Constants */
static const ioline_t button_Lines[Button_Num] = {
  LINE_BUTTON1, LINE_BUTTON2, LINE_BUTTON3, LINE_BUTTON4
};

/* Variables */
static virtual_timer_t button_vt;
static button_state_t button_States[Button_Num] = {0};
static uint8_t button_Time[Button_Num] = {0};

/* Functions */
static void button_Process(virtual_timer_t *vtp, void *p) {
  (void)vtp;
  (void)p;
  static int pause = 0;

  if (pause != 0) {
    pause --;
  } else {

    int i;
    for (i=0; i<Button_Num; i++) {

      if (palReadLine(button_Lines[(button_num_t)i]) == BUTTON_PRESSED) {
      /* button pressed */
        button_Time[i] ++;
        if (button_Time[i] >= (BTN_TIME_HOLDED/BTN_SCAN_PERIOD)) {
          /* process long press */
          button_Time[i] -= (BTN_TIME_REPEATED/BTN_SCAN_PERIOD);
          button_States[i] = BTN_st_Holded;
          btnh[i](BTN_st_Pressed); // autorepeat
        }

      } else if (button_Time[i] != 0) {
      /* button released */
        if (button_Time[i] >= (BTN_TIME_PRESSED/BTN_SCAN_PERIOD)) {
          /* process short press */
          button_States[i] = BTN_st_Pressed;
          btnh[i](BTN_st_Pressed);
        }
        button_Time[i] = 0;
        button_States[i] = BTN_st_Released;
        pause = (BTN_SCAN_PAUSE/BTN_SCAN_PERIOD);
        btnh[i](BTN_st_Pressed);
      }
    } /* end FOR */

  } /* end Pause check */
}

void buttons_Init(btn_hndlr ha[]) {
  btnh[Button1] = ha[Button1];
  btnh[Button2] = ha[Button2];
  btnh[Button3] = ha[Button3];
  btnh[Button4] = ha[Button4];

  chVTSetContinuous(&button_vt, TIME_MS2I(BTN_SCAN_PERIOD), button_Process, NULL);
}

button_state_t buttons_GetState(const button_num_t btn) {
  return button_States[btn];
}
